/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: EventComponent.java 2122 2008-09-27 09:41:35Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.cmi.component.event;

import java.util.LinkedList;

import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.event.CMIEventComponent;
import org.ow2.cmi.event.CMIEventListener;
import org.ow2.cmi.event.CMIManagedObject;
import org.ow2.cmi.rpc.CMIInvocationHandler;
import org.ow2.util.component.api.ComponentException;
import org.ow2.util.event.api.IEventDispatcher;
import org.ow2.util.event.api.IEventListener;
import org.ow2.util.event.api.IEventService;
import org.ow2.util.event.impl.EventService;

/**
 * Implementation of the CMI event component.
 * @author Loris Bouzonnet
 */
public class EventComponent implements CMIEventComponent {

    /**
     * The component event service.
     */
    private IEventService service;

    /**
     * The component event listeners.
     */
    private final LinkedList<IEventListener> listeners =
        new LinkedList<IEventListener>();

    /**
     * The component CMIManagedObject.
     */
    private final LinkedList<CMIManagedObject> objects =
        new LinkedList<CMIManagedObject>();

    /**
     * Set the event service to use.
     * @param service The event service to use.
     */
    public synchronized void setEventService(final IEventService service) {
        this.service = service;
    }

    /* (non-Javadoc)
     * @see org.ow2.util.component.api.IComponent#init()
     */
    public void init() throws ComponentException {
        //if nobody define the EventService instance (OSGi, ...), the component create its own instance.
        if (this.service == null) {
            this.service = new EventService();
            this.service.start();
        }
    }

    /* (non-Javadoc)
     * @see org.ow2.util.component.api.IComponent#start()
     */
    public void start() throws ComponentException {
        // Do nothing
    }

    /* (non-Javadoc)
     * @see org.ow2.util.component.api.IComponent#stop()
     */
    public void stop() throws ComponentException {
        for (IEventListener listener : this.listeners) {
            this.service.unregisterListener(listener);
        }
        for (CMIManagedObject object : this.objects) {
            this.service.unregisterDispatcher(object.getCMIManagedObjectId());
        }
        this.listeners.clear();
        objects.clear();
    }

    /**
     * Register a new event listener.<br>
     * The listener will automatically be register with each dispatcher matching his filter.
     * @param eventListener The listener to register.
     */
    public synchronized void registerEventListener(final CMIEventListener eventListener) {
        this.service.registerListener(eventListener, eventListener.getEventProviderFilter());
        this.listeners.add(eventListener);
    }

    /**
     * Unregister an event listener.
     * @param eventListener The listener to unregister.
     */
    public synchronized void unregisterEventListener(final CMIEventListener eventListener) {
        this.service.unregisterListener(eventListener);
        this.listeners.remove(eventListener);
    }

    /**
     * Register a new CMI managed object.<br>
     * If a CMI managed object with the same id is already registered, it will be unregistered first.
     * @param object The CMI managed object to register.
     * @param dispatcher The event dispatcher for this CMI managed object.
     */
    public synchronized void registerCMIManagedObject(final CMIManagedObject object, final IEventDispatcher dispatcher) {
        if (ClusterViewManager.class.isAssignableFrom(object.getClass())) {
            registerClusterViewManager((ClusterViewManager) object, dispatcher);
        } else if (CMIInvocationHandler.class.isAssignableFrom(object.getClass())) {
            registerCMIInvocationHandler((CMIInvocationHandler<?>) object, dispatcher);
        }
    }

    /**
     * Unregister a CMI managed object.
     * @param object The CMI managed object to unregister.
     */
    public synchronized void unregisterCMIManagedObject(final CMIManagedObject object) {
        if (ClusterViewManager.class.isAssignableFrom(object.getClass())) {
            unregisterClusterViewManager((ClusterViewManager) object);
        } else if (CMIInvocationHandler.class.isAssignableFrom(object.getClass())) {
            unregisterCMIInvocationHandler((CMIInvocationHandler<?>) object);
        }
    }

    /**
     * @param invocationHandler
     */
    private void unregisterCMIInvocationHandler(final CMIInvocationHandler<?> invocationHandler) {
        this.service.unregisterDispatcher(invocationHandler.getCMIManagedObjectId());
        this.objects.remove(invocationHandler);
    }

    /**
     * @param manager
     */
    private void unregisterClusterViewManager(final ClusterViewManager manager) {
        this.service.unregisterDispatcher(manager.getCMIManagedObjectId());
        this.objects.remove(manager);
    }

    private void registerClusterViewManager(final ClusterViewManager clusterViewManager, final IEventDispatcher dispatcher) {
        this.service.registerDispatcher(clusterViewManager.getCMIManagedObjectId(), dispatcher);
        this.objects.add(clusterViewManager);
    }

    private void registerCMIInvocationHandler(
            final CMIInvocationHandler<?> invocationHandler, final IEventDispatcher dispatcher) {
        this.service.registerDispatcher(invocationHandler.getCMIManagedObjectId(), dispatcher);
        this.objects.add(invocationHandler);
    }

}
